<?php
/**
 * Currency Exchange Rates - Functions
 *
 * @version 1.0.0
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! function_exists( 'alg_cer_get_saved_exchange_rate' ) ) {
	/**
	 * alg_cer_get_saved_exchange_rate.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_cer_get_saved_exchange_rate( $currency_from, $currency_to ) {
		$rates = get_option( 'alg_cer_rates', array() );
		return ( isset( $rates[ $currency_from ][ $currency_to ] ) ? $rates[ $currency_from ][ $currency_to ] : false );
	}
}

if ( ! function_exists( 'alg_cer_get_currency_exchange_rate_servers' ) ) {
	/**
	 * alg_cer_get_currency_exchange_rate_servers.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_cer_get_currency_exchange_rate_servers() {
		return apply_filters( 'alg_cer_currency_exchange_rate_servers', array(
			'fixer' => __( 'Fixer.io', 'wp-currency-exchange-rates' ),
		) );
	}
}

if ( ! function_exists( 'alg_cer_get_exchange_rate' ) ) {
	/*
	 * alg_cer_get_exchange_rate.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    (maybe) calculate by invert
	 * @todo    (maybe) offsets
	 * @todo    (maybe) rounding (up/down)
	 */
	function alg_cer_get_exchange_rate( $currency_from, $currency_to, $max_precision = 6 ) {
		$exchange_rates_server = get_option( 'alg_cer_server', 'yahoo' );
		switch ( $exchange_rates_server ) {
			case 'fixer':
				$return = alg_cer_fixer_get_exchange_rate( $currency_from, $currency_to );
				break;
			default:
				$return = apply_filters( 'alg_get_exchange_rate', false, $exchange_rates_server, $currency_from, $currency_to );
		}
		return ( false !== $return ? round( $return, $max_precision ) : false );
	}
}

if ( ! function_exists( 'alg_cer_download_url' ) ) {
	/*
	 * alg_cer_download_url.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    (maybe) use `download_url()` function
	 */
	function alg_cer_download_url( $url ) {
		$response = false;
		if ( function_exists( 'curl_version' ) ) {
			$curl = curl_init( $url );
			curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );
			$response = curl_exec( $curl );
			curl_close( $curl );
		} elseif ( ini_get( 'allow_url_fopen' ) ) {
			$response = file_get_contents( $url );
		}
		return $response;
	}
}

if ( ! function_exists( 'alg_cer_fixer_get_exchange_rate' ) ) {
	/*
	 * alg_cer_fixer_get_exchange_rate.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    get all rates at once (for "saved rates")
	 */
	function alg_cer_fixer_get_exchange_rate( $currency_from, $currency_to ) {
		if ( false !== ( $response = alg_cer_download_url( 'http://api.fixer.io/latest?base=' . $currency_from . '&symbols=' . $currency_to ) ) ) {
			$exchange_rate = json_decode( $response );
		}
		return ( isset( $exchange_rate->rates->{$currency_to} ) ) ? floatval( $exchange_rate->rates->{$currency_to} ) : false;
	}
}
